/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef HTML_SCRIPT_HH
#define HTML_SCRIPT_HH

#include "html/Attrib.hh"
#include "html/object.hh"
#include <string>

namespace html {

   /**  The script class defines an atomic html object containing a script
     *  string and a list of script-specific attributes. A script source file
     *  may be specified in lieu of the script text.
     *  @memo Html script object
     *  @author J. Zweizig
     *  @version 1.1; Modified April 4, 2019
     *  @ingroup IO_html
     */
   class script : public object {
   public:
      /**  Construct an empty script object.
        *  @memo Default constructor.
	*/
      script(void);

      /**  Construct a script object and set it to contain the specified 
        *  string data.
	*  @memo Data constructor
	*  @param x String containing the initial script.
	*/
      script(const std::string& x);

      /**  Construct a script object from a double precision float.
        *  @memo Construct numeric script.
	*  @param x Number to be inserted as script.
	*/
      script(double x);

      /**  Destroy a script object.
        *  @memo Script destructor.
	*/
      virtual ~script(void);

      /**  Create a script object identical to the existing instance and return
        *  a pointer to the result. This method overrides the default action 
	*  of the html object clone virtual method.
	*  @memo Clone a script object.
	*  @return pointer to an exact copy of the current script instance.
	*/
      script* clone(void) const;

      /**  Write the object script to the specified html writer stream.
        *  @memo Write the script.
	*  @param out Html write to which the script is written.
	*/
      void write(writer& out) const;

      /**  Replace any existing script with the specified string.
        *  @memo Set the script.
	*  @param txt String containg new script.
	*/ 
      void insertScript(const std::string& txt);

      /**  Append a specified string to any existing script on the object.
        *  @memo Append a script string.
	*  @param txt String to be added to the script.
	*/
      void addScript(const std::string& txt);

      /**  Append a specified string to any existing script in the object.
        *  @memo Append a string to the script.
	*  @param txt String to be added to the script.
	*  @return Reference to the updated script object.
	*/
      script& operator<<(const std::string& txt);

      /**  Set the location of the script source file.
        *  \brief Set the source location.
	*  \param path Location of the script.
	*  \return Reference to the script instance.
	*/
      void setSource(const std::string& path);

      /**  Specify the script type attribute.
        *  @memo Set the script type.
	*  @return Script type string.
	*  @param lang A script type (language) specifier.
	*/
      void setType(const std::string& lang);

   private:
      attList    mAttr;
      std::string mScript;
   };

}

#endif  // HTML_SCRIPT_HH
